@@ -38,7 +38,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <gnome.h>
+#include <unistd.h>
 #include "utils.h"
 #include "properties.h"
 
@@ -206,7 +206,7 @@ static GtkStockItem test_url_items [] =
 };
 
 void
-editor_check_stock ()
+editor_check_stock (void)
 {
 	if (!stock_test_url_added) {
 		GdkPixbuf *pixbuf;
@@ -227,8 +227,8 @@ editor_check_stock ()
 				gtk_icon_factory_add (factory, GTKHTML_STOCK_TEST_URL, test_url_iconset);
 				gtk_icon_factory_add_default (factory);
 			}
-			gtk_stock_add_static (test_url_items, G_N_ELEMENTS (test_url_items));
 		}
+		gtk_stock_add_static (test_url_items, G_N_ELEMENTS (test_url_items));
 		stock_test_url_added = TRUE;
 	}
 }
